# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2012  Luis Falcon <lfalcon@gnusolidario.org>
#    Copyright (C) 2011  Adrián Bernardi, Mario Puntin (health_invoice)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from trytond.model import ModelView, ModelSQL, fields, ModelSingleton
from trytond.pyson import Eval, Equal
from trytond.pool import Pool


class GnuHealthSequences(ModelSingleton, ModelSQL, ModelView):
    "Standard Sequences for GNU Health"

    _description = __doc__
    _name = "gnuhealth.sequences"

    health_service_sequence = fields.Property(fields.Many2One('ir.sequence',
        'Health Service Sequence', domain=[
            ('code', '=', 'gnuhealth.health_service')
        ], required=True))

GnuHealthSequences()


class HealthService(ModelSQL, ModelView):
    'Health Service'
    _name = 'gnuhealth.health_service'
    _description = __doc__

    name = fields.Char('ID', readonly=True)
    desc = fields.Char('Description', required=True)
    patient = fields.Many2One('gnuhealth.patient', 'Patient', required=True)
    service_date = fields.Date('Date')
    service_line = fields.One2Many('gnuhealth.health_service.line',
        'name', 'Service Line', help="Service Line")

    state = fields.Selection([
        ('draft', 'Draft'),
        ('invoiced', 'Invoiced'),
        ], 'State', readonly=True)

    def default_state(self):
        return 'draft'

    def button_set_to_draft(self, ids):
        self.write(ids, {'state': 'draft'})
        return True

    def __init__(self):
        super(HealthService, self).__init__()
        self._sql_constraints += [
            ('name_uniq', 'UNIQUE(name)', 'The Service ID must be unique')]

        self._rpc.update({'button_set_to_draft': True})

        self._buttons.update({
            'button_set_to_draft': {'invisible': Equal(Eval('state'),
                'draft')}
            })

    def create(self, values):
        sequence_obj = Pool().get('ir.sequence')
        config_obj = Pool().get('gnuhealth.sequences')

        values = values.copy()
        if not values.get('name'):
            config = config_obj.browse(1)
            values['name'] = sequence_obj.get_id(
            config.health_service_sequence.id)

        return super(HealthService, self).create(values)

HealthService()


class HealthServiceLine(ModelSQL, ModelView):
    'Health Service'
    _name = 'gnuhealth.health_service.line'
    _description = __doc__

    name = fields.Many2One('gnuhealth.health_service', 'Service', readonly=True)
    desc = fields.Char('Description', required=True)
    appointment = fields.Many2One('gnuhealth.appointment', 'Appointment',
        help='Enter or select the date / ID of the appointment related to'\
        ' this evaluation')

    to_invoice = fields.Boolean('Invoice')
    product = fields.Many2One('product.product', 'Product', required=True)
    qty = fields.Integer('Qty')
    from_date = fields.Date('From')
    to_date = fields.Date('To')

    def default_qty(self):
        return 1

HealthServiceLine()




# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2011-2012  Sebastián Marró <smarro@thymbra.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from datetime import timedelta

from trytond.model import ModelView, ModelSQL, fields
from trytond.pool import Pool


class Physician(ModelSQL, ModelView):
    "Add Calendar to Physician"
    _name = "gnuhealth.physician"
    _description = __doc__

    calendar = fields.Many2One('calendar.calendar', 'Calendar')

Physician()


class Appointment(ModelSQL, ModelView):
    'Add Calendar to the Appointment'
    _name = 'gnuhealth.appointment'
    _description = __doc__

    event = fields.Many2One('calendar.event', 'Calendar Event', readonly=True,
        help="Calendar Event")
    appointment_time = fields.Integer('Appointment Time',
        help='Appointment Time (Minutes)')

    def default_appointment_time(self):
        return 30

    def create(self, values):
        event_obj = Pool().get('calendar.event')
        patient_obj = Pool().get('gnuhealth.patient')
        physician_obj = Pool().get('gnuhealth.physician')

        if values['doctor']:
            doctor = physician_obj.browse(values['doctor'])
            if doctor.calendar:
                patient = patient_obj.browse(values['patient'])
                values['event'] = event_obj.create({
                    'dtstart': values['appointment_date'],
                    'dtend': values['appointment_date'] +
                        timedelta(minutes=values['appointment_time']),
                    'calendar': doctor.calendar.id,
                    'summary': patient.name.lastname + ', ' + patient.name.name,
                    })
        return super(Appointment, self).create(values)

    def write(self, ids, values):
        event_obj = Pool().get('calendar.event')
        patient_obj = Pool().get('gnuhealth.patient')
        physician_obj = Pool().get('gnuhealth.physician')

        if isinstance(ids, (int, long)):
            ids = [ids]
        for appointment_id in ids:
            appointment = self.browse(appointment_id)
            if appointment.event:
                if 'appointment_date' in values:
                    event_obj.write(appointment.event.id, {
                        'dtstart': values['appointment_date'],
                        'dtend': values['appointment_date'] +
                            timedelta(minutes=appointment.appointment_time),
                        })
                if 'appointment_time' in values:
                    event_obj.write(appointment.event.id, {
                        'dtend': appointment.appointment_date +
                            timedelta(minutes=values['appointment_time']),
                        })
                if 'doctor' in values:
                    doctor = physician_obj.browse(values['doctor'])
                    event_obj.write(appointment.event.id, {
                        'calendar': doctor.calendar.id,
                        })
                if 'patient' in values:
                    patient = patient_obj.browse(values['patient'])
                    event_obj.write(appointment.event.id, {
                        'summary': patient.name.name,
                        })
        return super(Appointment, self).write(ids, values)

    def delete(self, ids):
        event_obj = Pool().get('calendar.event')

        if isinstance(ids, (int, long)):
            ids = [ids]
        for appointment_id in ids:
            appointment = self.browse(appointment_id)
            if appointment.event:
                event_obj.delete(appointment.event.id)
        return super(Appointment, self).delete(ids)

Appointment()



# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2012  Luis Falcon <lfalcon@gnusolidario.org>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
import qrcode
import StringIO
from trytond.model import ModelView, ModelSQL, fields


# Add the QR field and QR image in the patient model

class Patient(ModelSQL, ModelView):
    'Patient'
    _name = 'gnuhealth.patient'

    def make_qrcode(self, ids, name):
# Create the QR code
        result = {}
        for patient_data in self.browse(ids):

            patient_ssn = patient_data.ssn or ''

            patient_blood_type = patient_data.blood_type or ''

            patient_rh = patient_data.rh or ''

            patient_sex = patient_data.sex or ''

            patient_dob = patient_data.dob or ''

            patient_id = patient_data.identification_code or ''

            if patient_data.lastname:
                patient_lastname = patient_data.lastname + ', '
            else:
                patient_lastname = ''

            qr_string = 'ID: ' + patient_id \
                + '\nName: ' + patient_lastname + ',' \
                    + patient_data.name.name \
                + '\nSSN: ' + patient_ssn \
                + '\nSex: ' + patient_sex \
                + '\nDoB: ' + str(patient_dob) \
                + '\nBlood Type: ' + patient_blood_type \
                    + ' ' + patient_rh

            qr_image = qrcode.make(qr_string)

# Make a PNG image from PIL without the need to create a temp file
            holder = StringIO.StringIO()
            qr_image.save(holder)
            qr_png = holder.getvalue()
            holder.close()

            result[patient_data.id] = buffer(qr_png)

        return result

# Add the QR Code to the Patient
    qr = fields.Function(fields.Binary('QR Code'), 'make_qrcode')

Patient()


# Add the QR code field and image to the Newborn

class Newborn(ModelSQL, ModelView):
    'NewBorn'
    _name = 'gnuhealth.newborn'

    def make_qrcode(self, ids, name):
# Create the QR code
        result = {}
        for newborn_data in self.browse(ids):

            if newborn_data.mother:
                if newborn_data.mother.name.lastname:
                    newborn_mother_lastname = newborn_data.mother.name.lastname + ', '
                else:
                    newborn_mother_lastname = ''

                newborn_mother_name = newborn_data.mother.name.name or ''

                newborn_mother_id = newborn_data.mother.identification_code or ''

            else:
                newborn_mother_lastname = ''
                newborn_mother_name = ''
                newborn_mother_id = ''
                
            newborn_name = newborn_data.name or ''

            newborn_sex = newborn_data.sex or ''
            
            newborn_birth_date = newborn_data.birth_date or ''

            qr_string = 'ID: ' + newborn_name \
                + '\nMother: ' + newborn_mother_lastname \
                        + newborn_mother_name \
                + '\nMother\'s ID: ' + newborn_mother_id \
                + '\nSex: ' + newborn_sex \
                + '\nDoB: ' + str(newborn_birth_date)

            qr_image = qrcode.make(qr_string)

# Make a PNG image from PIL without the need to create a temp file
            holder = StringIO.StringIO()
            qr_image.save(holder)
            qr_png = holder.getvalue()
            holder.close()

            result[newborn_data.id] = buffer(qr_png)

        return result

# Add the QR Code to the Newborn
    qr = fields.Function(fields.Binary('QR Code'), 'make_qrcode')

Newborn()



# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2012  Luis Falcon <lfalcon@gnusolidario.org>
#    
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from trytond.model import ModelView
from trytond.wizard import Wizard, StateTransition, StateView, StateTransition, \
    Button
from trytond.transaction import Transaction

from trytond.pool import Pool


class CreateServiceInvoiceInit(ModelView):
    'Create Service Invoice Init'
    _name = 'gnuhealth.service.invoice.init'
    _description = __doc__

CreateServiceInvoiceInit()


class CreateServiceInvoice(Wizard):
    'Create Service Invoice'
    _name = 'gnuhealth.service.invoice.create'

    start = StateView('gnuhealth.service.invoice.init',
        'health_services.view_health_service_invoice', [
            Button('Cancel', 'end', 'tryton-cancel'),
            Button('Create Invoice', 'create_service_invoice', 'tryton-ok', True),
            ])
    
    create_service_invoice = StateTransition()


    def transition_create_service_invoice(self, session):
        health_service_obj = Pool().get('gnuhealth.health_service')
        invoice_obj = Pool().get('account.invoice')
        party_obj = Pool().get('party.party')
        
        selected_services = health_service_obj.browse(Transaction().context.get('active_ids'))
        
        #Invoice Header
        for service in selected_services:

            if service.state == 'invoiced':
                    self.raise_user_error('duplicate_invoice')
                    
            invoice_data = {}

            invoice_data['description'] = service.desc
            invoice_data['party'] = service.patient.name.id  
            invoice_data['account'] = service.patient.name.account_receivable.id
            invoice_data['invoice_address'] = party_obj.address_get(service.patient.name.id, type='invoice')
            invoice_data['reference'] = service.name
           
            invoice_data['payment_term'] = \
                    service.patient.name.customer_payment_term and \
                    service.patient.name.customer_payment_term.id or \
                    False
            

        
            #Invoice Lines
            seq = 0
            invoice_lines = []
            
            for line in service.service_line:
                seq = seq + 1
                account = line['product'].template.account_revenue_used.id

                if line['to_invoice'] :                  
                    invoice_lines.append(('create', {
                            'product': line['product'].id,
                            'description': line['desc'],
                            'quantity': line['qty'],
                            'account': account,
                            'unit' : line['product'].default_uom.id,
                            'unit_price' : line['product'].list_price,
                            'sequence': seq
                        }))

                invoice_data['lines'] = invoice_lines
                                
            invoice_document = invoice_obj.create(invoice_data)
            
            
            # Change to invoiced the status on the service document.
            health_service_obj.write(service.id, {'state': 'invoiced'})
        
        return 'end'
        
    def __init__(self):
        super(CreateServiceInvoice, self).__init__()
        self._error_messages.update({
            'duplicate_invoice': 'Service already invoiced'})
            


CreateServiceInvoice()




# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2012  Luis Falcon <lfalcon@gnusolidario.org>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from trytond.model import ModelView, ModelSQL, fields
from trytond.pyson import Eval, Not, Bool
from trytond.pool import Pool


class PuerperiumMonitor(ModelSQL, ModelView):
    'Puerperium Monitor'
    _name = 'gnuhealth.puerperium.monitor'
    _description = __doc__

    name = fields.Many2One('gnuhealth.patient', 'Patient ID')
    date = fields.DateTime('Date and Time', required=True)
    systolic = fields.Integer('Systolic Pressure')
    diastolic = fields.Integer('Diastolic Pressure')
    frequency = fields.Integer('Heart Frequency')

    lochia_amount = fields.Selection([
        ('n', 'normal'),
        ('e', 'abundant'),
        ('h', 'hemorrhage'),
        ], 'Lochia amount', select=True)

    lochia_color = fields.Selection([
        ('r', 'rubra'),
        ('s', 'serosa'),
        ('a', 'alba'),
        ], 'Lochia color', select=True)

    lochia_odor = fields.Selection([
        ('n', 'normal'),
        ('o', 'offensive'),
        ], 'Lochia odor', select=True)

    uterus_involution = fields.Integer('Fundal Height',
        help="Distance between the symphysis pubis and the uterine fundus " \
        "(S-FD) in cm")
    temperature = fields.Float('Temperature')

PuerperiumMonitor()


class PerinatalMonitor(ModelSQL, ModelView):
    'Perinatal monitor'
    _name = 'gnuhealth.perinatal.monitor'
    _description = __doc__

    name = fields.Many2One('gnuhealth.patient', 'Patient ID')
    date = fields.DateTime('Date and Time')
    systolic = fields.Integer('Systolic Pressure')
    diastolic = fields.Integer('Diastolic Pressure')
    contractions = fields.Integer('Contractions')
    frequency = fields.Integer('Mother\'s Heart Frequency')
    dilation = fields.Integer('Cervix dilation')
    f_frequency = fields.Integer('Fetus Heart Frequency')
    meconium = fields.Boolean('Meconium')
    bleeding = fields.Boolean('Bleeding')
    fundal_height = fields.Integer('Fundal Height')
    fetus_position = fields.Selection([
        ('n', 'Correct'),
        ('o', 'Occiput / Cephalic Posterior'),
        ('fb', 'Frank Breech'),
        ('cb', 'Complete Breech'),
        ('t', 'Transverse Lie'),
        ('t', 'Footling Breech'),
        ], 'Fetus Position', select=True)

PerinatalMonitor()


class Perinatal(ModelSQL, ModelView):
    'Perinatal Information'
    _name = 'gnuhealth.perinatal'
    _description = __doc__

    name = fields.Many2One('gnuhealth.patient', 'Patient ID')
    admission_code = fields.Char('Code')
    gravida_number = fields.Integer('Gravida #')
    abortion = fields.Boolean('Abortion')
    admission_date = fields.DateTime('Admission',
        help="Date when she was admitted to give birth", required=True)
    prenatal_evaluations = fields.Integer('Prenatal evaluations',
        help="Number of visits to the doctor during pregnancy")
    start_labor_mode = fields.Selection([
        ('n', 'Normal'),
        ('i', 'Induced'),
        ('c', 'c-section'),
        ], 'Labor mode', select=True)
    gestational_weeks = fields.Integer('Weeks')
    gestational_days = fields.Integer('Days')
    fetus_presentation = fields.Selection([
        ('n', 'Correct'),
        ('o', 'Occiput / Cephalic Posterior'),
        ('fb', 'Frank Breech'),
        ('cb', 'Complete Breech'),
        ('t', 'Transverse Lie'),
        ('t', 'Footling Breech'),
        ], 'Fetus Presentation', select=True)
    dystocia = fields.Boolean('Dystocia')
    placenta_incomplete = fields.Boolean('Incomplete Placenta')
    placenta_retained = fields.Boolean('Retained Placenta')
    episiotomy = fields.Boolean('Episiotomy')
    vaginal_tearing = fields.Boolean('Vaginal tearing')
    forceps = fields.Boolean('Use of forceps')
    monitoring = fields.One2Many('gnuhealth.perinatal.monitor', 'name',
        'Monitors')
    puerperium_monitor = fields.One2Many('gnuhealth.puerperium.monitor', 'name',
        'Puerperium monitor')
    medication = fields.One2Many('gnuhealth.patient.medication', 'name',
        'Medication and anesthesics')
    dismissed = fields.DateTime('Discharged')
    place_of_death = fields.Selection([
        ('ho', 'Hospital'),
        ('dr', 'At the delivery room'),
        ('hh', 'in transit to the hospital'),
        ('th', 'Being transferred to other hospital'),
        ], 'Place of Death', help="Place where the mother died",
        states={'invisible': Not(Bool(Eval('mother_deceased')))},
        depends=['mother_deceased'])

    mother_deceased = fields.Boolean('Maternal death',
        help="Mother died in the process")

    notes = fields.Text('Notes')

Perinatal()


class GnuHealthPatient(ModelSQL, ModelView):
    'Add to the Medical patient_data class (gnuhealth.patient) the ' \
    'gynecological and obstetric fields.'
    _name = 'gnuhealth.patient'
    _description = __doc__

    currently_pregnant = fields.Boolean('Currently Pregnant')
    fertile = fields.Boolean('Fertile',
        help="Check if patient is in fertile age")
    menarche = fields.Integer('Menarche age')
    menopausal = fields.Boolean('Menopausal')
    menopause = fields.Integer('Menopause age')
    mammography = fields.Boolean('Mammography',
        help="Check if the patient does periodic mammographys")
    mammography_last = fields.Date('Last mammography',
        help="Enter the date of the last mammography")
    breast_self_examination = fields.Boolean('Breast self-examination',
        help="Check if patient does and knows how to self examine her breasts")
    pap_test = fields.Boolean('PAP test',
        help="Check if patient does periodic cytologic pelvic smear screening")
    pap_test_last = fields.Date('Last PAP test',
        help="Enter the date of the last Papanicolau test")
    colposcopy = fields.Boolean('Colposcopy',
        help="Check if the patient has done a colposcopy exam")
    colposcopy_last = fields.Date('Last colposcopy',
        help="Enter the date of the last colposcopy")

    gravida = fields.Integer('Gravida', help="Number of pregnancies")
    premature = fields.Integer('Premature', help="Premature Deliveries")
    abortions = fields.Integer('Abortions')
    full_term = fields.Integer('Full Term', help="Full term pregnancies")
    gpa = fields.Char('GPA',
        help="Gravida, Para, Abortus Notation. For example G4P3A1 : 4 " \
        "Pregnancies, 3 viable and 1 abortion")
    born_alive = fields.Integer('Born Alive')
    deaths_1st_week = fields.Integer('Deceased during 1st week',
        help="Number of babies that die in the first week")
    deaths_2nd_week = fields.Integer('Deceased after 2nd week',
        help="Number of babies that die after the second week")

    perinatal = fields.One2Many('gnuhealth.perinatal', 'name', 'Perinatal Info')

    menstrual_history = fields.One2Many('gnuhealth.patient.menstrual_history',
        'name', 'Menstrual History')

    mammography_history = fields.One2Many( \
        'gnuhealth.patient.mammography_history', 'name', 'Mammography History')

    pap_history = fields.One2Many('gnuhealth.patient.pap_history', 'name',
        'PAP smear History')

    colposcopy_history = fields.One2Many('gnuhealth.patient.colposcopy_history',
        'name', 'Colposcopy History')

GnuHealthPatient()


class PatientMenstrualHistory(ModelSQL, ModelView):
    'Menstrual History'
    _name = 'gnuhealth.patient.menstrual_history'
    _description = __doc__

    name = fields.Many2One('gnuhealth.patient', 'Patient', readonly=True,
        required=True)
    evaluation = fields.Many2One('gnuhealth.patient.evaluation', 'Evaluation',
        readonly=True)
    evaluation_date = fields.Date('Date', help="Evaluation Date", required=True)
    lmp = fields.Date('LMP', help="Last Menstrual Period", required=True)
    lmp_length = fields.Integer('Length', required=True)
    is_regular = fields.Boolean('Regular')
    dysmenorrhea = fields.Boolean('Dysmenorrhea')
    frequency = fields.Selection([
        ('amenorrhea', 'amenorrhea'),
        ('oligomenorrhea', 'oligomenorrhea'),
        ('eumenorrhea', 'eumenorrhea'),
        ('polymenorrhea', 'polymenorrhea'),
        ], 'frequency', sort=False)

    volume = fields.Selection([
        ('hypomenorrhea', 'hypomenorrhea'),
        ('normal', 'normal'),
        ('menorrhagia', 'menorrhagia'),
        ], 'volume', sort=False)

    def default_evaluation_date(self):
        return Pool().get('ir.date').today()

    def default_frequency(self):
        return 'eumenorrhea'

    def default_volume(self):
        return 'normal'

PatientMenstrualHistory()


class PatientMammographyHistory(ModelSQL, ModelView):
    'Mammography History'
    _name = 'gnuhealth.patient.mammography_history'
    _description = __doc__

    name = fields.Many2One('gnuhealth.patient', 'Patient', readonly=True,
        required=True)
    evaluation = fields.Many2One('gnuhealth.patient.evaluation', 'Evaluation',
        readonly=True)
    evaluation_date = fields.Date('Date', help=" Date")
    last_mammography = fields.Date('Date', help="Last Mammography",
        required=True)
    result = fields.Selection([
        ('normal', 'normal'),
        ('abnormal', 'abnormal'),
        ], 'result', help="Please check the lab test results if the module is \
            installed", sort=False)

    comments = fields.Char('Remarks')

    def default_evaluation_date(self):
        return Pool().get('ir.date').today()

    def default_last_mammography(self):
        return Pool().get('ir.date').today()


PatientMammographyHistory()


class PatientPAPHistory(ModelSQL, ModelView):
    'PAP Test History'
    _name = 'gnuhealth.patient.pap_history'
    _description = __doc__

    name = fields.Many2One('gnuhealth.patient', 'Patient', readonly=True,
        required=True)
    evaluation = fields.Many2One('gnuhealth.patient.evaluation', 'Evaluation',
        readonly=True)
    evaluation_date = fields.Date('Date', help=" Date")
    last_pap = fields.Date('Date', help="Last Papanicolau", required=True)
    result = fields.Selection([
        ('negative', 'Negative'),
        ('c1', 'ASC-US'),
        ('c2', 'ASC-H'),
        ('g1', 'ASG'),
        ('c3', 'LSIL'),
        ('c4', 'HSIL'),
        ('g4', 'AIS'),
        ], 'result', help="Please check the lab results if the module is \
            installed", sort=False)

    comments = fields.Char('Remarks')

    def default_evaluation_date(self):
        return Pool().get('ir.date').today()

    def default_last_pap(self):
        return Pool().get('ir.date').today()

PatientPAPHistory()


class PatientColposcopyHistory(ModelSQL, ModelView):
    'Colposcopy History'
    _name = 'gnuhealth.patient.colposcopy_history'
    _description = __doc__

    name = fields.Many2One('gnuhealth.patient', 'Patient', readonly=True,
        required=True)
    evaluation = fields.Many2One('gnuhealth.patient.evaluation', 'Evaluation',
        readonly=True)
    evaluation_date = fields.Date('Date', help=" Date")
    last_colposcopy = fields.Date('Date', help="Last colposcopy", required=True)
    result = fields.Selection([
        ('normal', 'normal'),
        ('abnormal', 'abnormal'),
        ], 'result', help="Please check the lab test results if the module is \
            installed", sort=False)

    comments = fields.Char('Remarks')

    def default_evaluation_date(self):
        return Pool().get('ir.date').today()

    def default_last_colposcopy(self):
        return Pool().get('ir.date').today()

PatientColposcopyHistory()



# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    MODULE : Procedure Coding System .- ICD10-PCS
#    Copyright (C) 2008-2012  Luis Falcon <lfalcon@gnusolidario.org>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
{

	'name' : 'GNU Health : Procedure Coding System .- ICD10-PCS',  
    'version': '1.6.3',
	'author' : 'GNU Solidario',
	'email' : 'health@gnusolidario.org',
	'depends' : ['health'],
	'website' : "http://health.gnu.org",
    'name_es_ES': 'GNU Health : Facturación',
    
	'description' : """

Procedure Coding System for Medical : ICD-10-PCS

The ICD-10 Procedure Coding System (ICD-10-PCS) is system of medical classification used for procedural codes. The National Center for Health Statistics (NCHS) received permission from the World Health Organization (WHO) (the body responsible for publishing the International Classification of Diseases [ICD]) to create the ICD-10-PCS as a successor to Volume 3 of ICD-9-CM and a clinical modification of the original ICD-10.

Each code consists of seven alphanumeric characters. The second through seventh characters mean the same thing within each section, but may mean different things in other sections. Each character can be any of 34 possible values the ten digits 0-9 and the 24 letters A-H,J-N and P-Z may be used in each character. The letters O and I excluded to avoid confusion with the numbers 0 and 1. There are no decimals in ICD-10-PCS

Check http://en.wikipedia.org/wiki/ICD-10_Procedure_Coding_System

""",
    'description_es_ES' : '''

ICD-10-PCS es el acrónimo del International Classification of Diseases 10th Revision Procedure Classification System, en español Sistema de Codificación de Procedimientos anexo a la Codificación Internacional de enfermedades, 10 edición, un sistema de clasificación propuesto por los Centros de Servicios Medicare y Medicaid (CMS) de los Estados Unidos como anexo a la CIE-10,
 que estipula reglas de codificación especializadas para todos los procedimientos relacionados con la salud, usando un código alfanumérico de siete caracteres que provee una clave única para cada uno de ellos. El sistema estuvo en una fase de desarrollo durante más de cinco años y comenzó a utilizarse en 1998.
Reemplaza el volumen 3 de la CIE-9-MC.

Ver : http://es.wikipedia.org/wiki/ICD-10-PCS
''',

	"xml" : ["data/icd_10_pcs_2009_part1.xml","data/icd_10_pcs_2009_part2.xml","data/icd_10_pcs_2009_part3.xml"],

	"active": False 
}



# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    MODULE : Millennium Development Goal #6 HIV/AIDS, Malaria and Tuberculosis
#    Copyright (C) 2008-2012  Luis Falcon <lfalcon@gnusolidario.org>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
{

    'name': 'GNU Health : MDG 6. HIV/AIDS, Malaria and Tuberculosis',
    'version': '1.6.3',
    'author': 'GNU Solidario',
    'email': 'health@gnusolidario.org',
    'website': "http://health.gnu.org",
    'depends': ['health','health_lab'],
    'name_es_ES': 'GNU Health: MDG 6. VIH/SIDA, Malaria y Tuberculosis',

    "xml": ["data/lab_test_data.xml"],

    'description': """

Millennium Development Goal # 6
--------------------------------

health_mdg6 is a GNU Health module that incorporates the Millennium Development Goal # 6, to combat HIV/AIDS, malaria and Tuberculosis

The United Nations signed in September 2000 the Millennium Declaration, to fight poverty, hunger, disease, illiteracy, environmental degradation, and discrimination against women .


The Millennium Development Goals (MDG) has derived from the Millennium declaration and it has 8 goals. MDG 6 focuses on HIV/AIDS, Malaria and Tuberculosis.

GNU Solidario believes that Free Software is a key factor to accomplish these goals. GNU Health, the Free Health and Hospital Information System will help the health centers, researches, NGOs and professionals and volunteers to eradicate these devastating diseases.

GNU Health will incorporate the specific functionality to help prevent, diagnose, treat and control each of these devastating diseases.

The health_mdg6 module will maintain the multi-disciplinary approach, providing tools to the social worker, nurse, biochemist, laboratory , pharmacist, epidemiologist and physician, integrating the health professional into the community.

So, for example, upon a new possible case of active Tuberculosis (TB), GNU Health will execute the following actions : Detect and notify the contacts of the patient; Start profilaxis treatment on contacts ; Create a new laboratory order for culture and antibiogram and check for the strain resistance ; verify the DOTS (Direct Observed Treatment Short) on the patient ; Check the stock for the PPD test-kits and do a forecast for the existing anti-tuberculosis medicaments; Check for availability of special rooms if there is need for hospitalization; inform the health authorities of a new case; start a new prevention campaign.

""",

    'description_es_ES': '''
Acerca de GNU Health
---------------------------------

GNU Health es un sistema libre de Gestión Hospitalaria y de Información de salud que ofrece las siguientes funciones :

    * Expediente Médico Electrónico (EMR)

    * Sistema de Gestión Hospitalaria (HIS)

    * Sistema de Información de Salud

Nuestro objetivo es contribuir con los profesionales de la salud alrededor del mundo para mejorar la calidad de vida de los más necesitados, ofreciendo un sistema libre que optimice la promoción de la salud y la prevención de la enfermedad.


Características de GNU Health :


    - Focalizado en medicina familiar y APS (Atención Primaria de la Salud)

    - Interés en condiciones Socio-economicas (estilos de vida, ámbito familiar, educación...)

    - Enfermedades y Procedimientos Médicos standard (ICD-10 / ICD-10-PCS)

    - Gestión de Internación (Hospitalización)

    - Marcadores genéticos y riesgos hereditarios: Más de 4200 genes relacionados con enfermedades

    - Epidemiología y otros registros estadísticos

    - Registro Electrónico. Sin necesidad de papel

    - Recetas

    - Facturación

    - Administración del Paciente (creación, evaluación / consultas, historia ...)

    - Administración de Laboratorio

    - Vademécum

    - Gestión de Stock y de cadena de abastecimiento

    - Administración Financiera y gestión Hospitalaria

    - Diseñado con los estándares de la industria en mente

    - Software Libre:  License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>

** Para mayor información, visite la página del proyecto en http://health.gnu.org

** Puede escribirnos a health@gnusolidario.org

''',


}



# -*- coding: utf-8 -*-
#    Copyright (C) 2008-2012  Luis Falcon
#    Copyright (C) 2012-2012  Sebastián Marró

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import datetime as dt
from datetime import timedelta, datetime
import random
import linecache

from proteus import config, Model, Wizard

config = config.set_trytond('gnuhealth_demo', database_type='postgresql', 
    user='admin', password='admin')

def RandomDate(start, end):
    """
    This function will return a random datetime between two datetime 
    objects.
    """
    delta = end - start
    int_delta = (delta.days * 24 * 60 * 60) + delta.seconds
    random_second = random.randrange(int_delta)
    return (start + timedelta(seconds=random_second))

def InitDatabase():
    Module = Model.get('ir.module.module')
    health_profile, = Module.find([('name', '=', 'health_profile')])
    Module.install([health_profile.id], config.context)
    Wizard('ir.module.module.install_upgrade').execute('upgrade')

def LoadBetzFamilyInfo():
    Party = Model.get('party.party')
    User = Model.get('res.user')
    Family = Model.get('gnuhealth.family')
    FamilyMember = Model.get('gnuhealth.family_member')
    Patient = Model.get('gnuhealth.patient')
    Physician = Model.get('gnuhealth.physician')
    MedicalSpecialty = Model.get('gnuhealth.specialty')
    Occupation = Model.get('gnuhealth.occupation')
    Pathology = Model.get('gnuhealth.pathology')
    PatientDiseaseInfo = Model.get('gnuhealth.patient.disease')
    PatientMedication = Model.get('gnuhealth.patient.medication')
    MedicationTemplate = Model.get('gnuhealth.medication.template')
    Medicament = Model.get('gnuhealth.medicament')
    FamilyDiseases = Model.get('gnuhealth.patient.family.diseases')
    PatientGeneticRisk = Model.get('gnuhealth.patient.genetic.risk')
    DiseaseGene = Model.get('gnuhealth.disease.gene')
    Newborn = Model.get('gnuhealth.newborn')

    party = Party()
    party.name = 'GNU SOLIDARIO Hospital'
    party.is_institution = True
    party.save()

    party = Party()
    party.name = 'Insurator'
    party.is_insurance_company = True
    party.save()

    party = Party()
    party.name = 'Cameron'
    party.lastname = 'Cordara'
    party.is_doctor = True
    party.is_person = True
    party.internal_user, = User.find([('login', '=', 'admin')])
    party.save()

    physician = Physician()
    physician.name = party
    physician.specialty, = MedicalSpecialty.find([('code', '=', 'GP')])
    physician.code = '765870'
    physician.save()

    party = Party()
    party.name = 'John'
    party.lastname = 'Betz'
    party.ref = '55576584'
    party.is_patient = True
    party.is_person = True
    party.save()

    party = Party()
    party.name = 'Ana'
    party.lastname = 'Betz'
    party.ref = '55567890'
    party.is_patient = True
    party.is_person = True
    party.save()

    family = Family()
    family.name = 'Betz family'
    family.save()

    FamilyMember(name=family, party=party, role='Mother').save()

    family_member = FamilyMember()
    family_member.name = family
    family_member.party, = Party.find([('ref', '=', '55576584')])
    family_member.role = 'Father'
    family_member.save()

    patient = Patient()
    patient.name = party
    patient.sex = 'f'
    patient.dob = datetime.strptime('10/4/1985', '%m/%d/%Y')
    patient.marital_status = 'm'
    patient.primary_care_doctor = physician
    patient.occupation, = Occupation.find([('name', '=', 'Teacher')])
    patient.ses = '2'
    patient.housing = '2'
    patient.education = '5'
    patient.critical_info = 'β-lactam hypersensitivity'
    patient.gpa = 'G1P1A0'
    patient.ex_smoker = True
    patient.sexual_preferences = 'h'
    patient.sexual_partners = 'm'
    patient.sexual_practices = 's'
    patient.save()

    patient_disease = PatientDiseaseInfo()
    patient_disease.name = patient
    patient_disease.pathology, = Pathology.find([
        ('name', '=', 'Insulin-dependent diabetes mellitus'),
    ])
    patient_disease.diagnosed_date = datetime.strptime('11/10/1993', '%m/%d/%Y')
    patient_disease.save()

    medication_template = MedicationTemplate()
    medication_template.medicament, = Medicament.find([
        ('name.name', '=', 'insulin injection (soluble)'),
    ])
    medication_template.indication, = Pathology.find([
        ('name', '=', 'Insulin-dependent diabetes mellitus'),
    ])
    medication_template.start_treatment = datetime.strptime('11/10/1993', '%m/%d/%Y')
    medication_template.save()

    patient_medication = PatientMedication()
    patient_medication.template = medication_template
    patient_medication.name = patient
    patient_medication.doctor = physician
    patient_medication.is_active = True
    patient_medication.diagnosed_date = datetime.strptime('11/10/1993', '%m/%d/%Y')
    patient_medication.save()

    family_diseases = FamilyDiseases()
    family_diseases.patient = patient
    family_diseases.name, = Pathology.find([('name', '=', "Marfan's syndrome")])
    family_diseases.xory = 'm'
    family_diseases.relative = 'grandfather'
    family_diseases.save()

    family_diseases = FamilyDiseases()
    family_diseases.patient = patient
    family_diseases.name, = Pathology.find([
        ('name', '=', 'Essential (primary) hypertension'),
    ])
    family_diseases.xory = 'f'
    family_diseases.relative = 'father'
    family_diseases.save()

    patient_genetic_risk = PatientGeneticRisk()
    patient_genetic_risk.patient = patient
    patient_genetic_risk.disease_gene, = DiseaseGene.find([('name', '=', 'BRCA1')])
    patient_genetic_risk.save()

    newborn = Newborn()
    newborn.mother = patient
    newborn.newborn_name = 'Matt'
    newborn.birth_date = datetime.strptime('3/15/2010', '%m/%d/%Y')
    newborn.sex = 'm'
    newborn.save()

def LoadRandomPatients():
    Party = Model.get('party.party')
    Patient = Model.get('gnuhealth.patient')
    Physician = Model.get('gnuhealth.physician')
    MedicalSpecialty = Model.get('gnuhealth.specialty')
    PatientEvaluation = Model.get('gnuhealth.patient.evaluation')
    Pathology = Model.get('gnuhealth.pathology')

    TODAY = dt.date.today()
    PATIENT_COUNT = 500
    PHYSICIAN_COUNT = 10
    SPECIALTY_COUNT = 59
    PATHOLOGY_COUNT = 14181
    for i in range(1, PATIENT_COUNT):
        party = Party()
        party.name = 'Name'
        party.ref = str(i+55000000)
        party.is_patient = True
        party.is_person = True
        party.save()

        patient = Patient()
        patient.name = party
        patient.sex = random.choice(['m', 'f'])
        patient.dob = RandomDate(datetime.strptime('1/1/1925', '%m/%d/%Y'), 
            datetime.strptime('1/1/2012', '%m/%d/%Y'))
        patient.save()

        if patient.sex == 'm':
            party.name = linecache.getline('boy_names', 
                random.randint(1, 156)).rstrip()
            party.lastname = linecache.getline('boy_names', 
                random.randint(1, 156)).rstrip()
        if patient.sex == 'f':
            party.name = linecache.getline('girl_names', 
                random.randint(1, 102)).rstrip()
            party.lastname = linecache.getline('girl_names', 
                random.randint(1, 102)).rstrip()
        party.save()

    for i in range(1, PHYSICIAN_COUNT):
        party = Party()
        party.name = 'Name'
        party.is_doctor = True
        party.is_person = True
        party.save()

        physician = Physician()
        physician.name = party
        physician.specialty, = MedicalSpecialty.find([
            ('id', '=', random.randint(1, SPECIALTY_COUNT))
        ])
        physician.code = str(i+700000)
        physician.save()

        if patient.sex == 'm':
            party.name = linecache.getline('boy_names', 
                random.randint(1, 156)).rstrip()
            party.lastname = linecache.getline('boy_names', 
                random.randint(1, 156)).rstrip()
        if patient.sex == 'f':
            party.name = linecache.getline('girl_names', 
                random.randint(1, 102)).rstrip()
            party.lastname = linecache.getline('girl_names', 
                random.randint(1, 102)).rstrip()
        party.save()

    for patient in Patient.find():
        for year in range(TODAY.year-5,TODAY.year):
            evaluation = PatientEvaluation()
            evaluation.patient = patient
            evaluation.evaluation_start = RandomDate(
                datetime.strptime('1/1/'+str(year), '%m/%d/%Y'), 
                datetime.strptime('12/31/'+str(year), '%m/%d/%Y'))
            evaluation.evaluation_endtime = evaluation.evaluation_start + \
                timedelta(minutes=30)
            evaluation.diagnosis, = Pathology.find([
                ('id', '=', random.randint(1, PATHOLOGY_COUNT))
            ])
            evaluation.doctor, = Physician.find([
                ('id', '=', random.randint(1, PHYSICIAN_COUNT))
            ])
            evaluation.specialty, = MedicalSpecialty.find([
                ('id', '=', random.randint(1, SPECIALTY_COUNT))
            ])
            evaluation.save()

if __name__ == '__main__':
    InitDatabase()
    LoadBetzFamilyInfo()
    LoadRandomPatients()



# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    MODULE : Profile
#    Copyright (C) 2008-2012  Luis Falcon <lfalcon@gnusolidario.org>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
{

    'name' : 'GNU Health : Installer',  
    'version': '1.6.3',
    'author' : 'GNU Solidario',
    'email' : 'health@gnusolidario.org',
    'website' : "http://health.gnu.org",
    'category' : 'Generic Modules/Others',
    'depends' : ['health','health_socioeconomics','health_lifestyle',
        'health_genetics','health_icd10','health_gyneco',
        'health_pediatrics','health_surgery','health_lab', 'health_inpatient'],
    'name_es_ES': 'GNU Health : Perfil de Instalación estándar',
   
    'description' : """


Profile for Main Modules.
- Core
- Socioeconomics
- Lifestyle
- Genetics
- Surgery
- Laboratory
- Pediatrics
- ICD-10 Coding
- Gynecology and Obstetrics
- Inpatient

We don't include in the main installation the following modules :
- ICD-10-PCS (Surgical Procedures)
- Invoicing (Billing)

You can install these two manually.


""",

    'description_es_ES': 
'''
Perfil de Instalación de la functionalidad estándar

- Núcleo
- Socioeconomía
- Estilos de vida
- Genetica
- Cirugía
- Laboratorio
- Pediatría 
- ICD-10 
- Ginecología y Obstetricia
- Hospitalización

Los módulos :

- ICD-10-PCS (Procedimientos quirúrgicos
- Facturación (Billing)

No están en el perfil de instalación estándar, pero sí en el sistema.
Si lo desea, los puede instalar a posteriori.
''',

    'xml': [],
    "active": False 
}



# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2012  Luis Falcon <lfalcon@gnusolidario.org>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from trytond.model import ModelView, ModelSQL, fields


class Surgery(ModelSQL, ModelView):
    'Surgery Functionality'
    _name = 'gnuhealth.surgery'
    _description = __doc__

    patient = fields.Many2One('gnuhealth.patient', 'Patient ID')
    admission = fields.Many2One('gnuhealth.appointment', 'Admission')
    operating_room = fields.Many2One('gnuhealth.hospital.or', 'Operating Room')
    code = fields.Char('Code', required=True, help="Health Center Unique code")
    procedures = fields.One2Many('gnuhealth.operation', 'name', 'Procedures',
        help="List of the procedures in the surgery. Please enter the first " \
        "one as the main procedure")
    pathology = fields.Many2One('gnuhealth.pathology', 'Base condition',
        help="Base Condition / Reason")
    classification = fields.Selection([
        ('o', 'Optional'),
        ('r', 'Required'),
        ('u', 'Urgent'),
        ], 'Classification', sort=False)
    surgeon = fields.Many2One('gnuhealth.physician', 'Surgeon',
        help="Surgeon who did the procedure")
    anesthetist = fields.Many2One('gnuhealth.physician', 'Anesthetist',
        help="Anesthetist in charge")
    date = fields.DateTime('Date')
    age = fields.Char('Patient age',
        help="Patient age at the moment of the surgery. Can be estimative")
    description = fields.Char('Description', required = True)
    preop_mallampati = fields.Selection([
        ('Class 1', 'Class 1: Full visibility of tonsils, uvula and soft palate'),
        ('Class 2', 'Class 2: Visibility of hard and soft palate, ' \
                    'upper portion of tonsils and uvula'),
        ('Class 3', 'Class 3: Soft and hard palate and base of the uvula are visible'),
        ('Class 4', 'Class 4: Only Hard Palate visible'),
        ], 'Mallampati Score', sort=False)
    preop_bleeding_risk = fields.Boolean('Risk of Massive bleeding',
        help="Check this box if patient has a risk of loosing more than 500 " \
        "ml in adults of over 7ml/kg in infants. If so, make sure that " \
        "intravenous access and fluids are available")
    preop_oximeter = fields.Boolean('Pulse Oximeter in place',
        help="Check this box when verified the pulse oximeter is in place " \
        "and functioning")
    preop_site_marking = fields.Boolean('Surgical Site Marking',
        help="The surgeon has marked the surgical incision")
    preop_antibiotics = fields.Boolean('Antibiotic Prophylaxis',
        help="Prophylactic antibiotic treatment within the last 60 minutes")
    preop_sterility = fields.Boolean('Sterility confirmed',
        help="Nursing team has confirmed sterility of the devices and room")
    extra_info = fields.Text('Extra Info')

Surgery()


class MedicalOperation(ModelSQL, ModelView):
    'Operation - Surgical Procedures'
    _name = 'gnuhealth.operation'
    _description = __doc__

    name = fields.Many2One('gnuhealth.surgery', 'Surgery')
    procedure = fields.Many2One('gnuhealth.procedure', 'Code', required=True,
        select=True,
        help="Procedure Code, for example ICD-10-PCS Code 7-character string")
    notes = fields.Text('Notes')

MedicalOperation()


class MedicalPatient(ModelSQL, ModelView):
    'Add to the Medical patient_data class (medical.patient) the surgery ' \
    'field.'
    _name = 'gnuhealth.patient'
    _description = __doc__

    surgery = fields.One2Many('gnuhealth.surgery', 'patient', 'Surgeries')

MedicalPatient()



# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2011-2012  Sebastián Marró <smarro@thymbra.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from trytond.model import ModelView, ModelSQL, fields
from trytond.pool import Pool


class HospitalBed(ModelSQL, ModelView):
    "Add Calendar to Hospital Bed"
    _name = "gnuhealth.hospital.bed"
    _description = __doc__

    calendar = fields.Many2One('calendar.calendar', 'Calendar')

HospitalBed()


class InpatientRegistration(ModelSQL, ModelView):
    'Add Calendar to the Inpatient Registration'
    _name = 'gnuhealth.inpatient.registration'
    _description = __doc__

    event = fields.Many2One('calendar.event', 'Calendar Event', readonly=True,
        help="Calendar Event")

    def button_registration_confirm(self, ids):
        super(InpatientRegistration, self).button_registration_confirm(ids)

        event_obj = Pool().get('calendar.event')

        for inpatient_registration_id in ids:
            inpatient_registration = self.browse(inpatient_registration_id)
            if inpatient_registration.bed.calendar:
                if not inpatient_registration.event:
                    event_id = event_obj.create({
                        'dtstart': inpatient_registration.hospitalization_date,
                        'dtend': inpatient_registration.discharge_date,
                        'calendar': inpatient_registration.bed.calendar.id,
                        'summary': inpatient_registration.patient.name.name
                        })
                    self.write(inpatient_registration_id, {'event': event_id})

        return True

    def button_patient_discharge(self, ids):
        super(InpatientRegistration, self).button_patient_discharge(ids)

        event_obj = Pool().get('calendar.event')

        for inpatient_registration_id in ids:
            inpatient_registration = self.browse(inpatient_registration_id)
            if inpatient_registration.event:
                event_obj.delete(inpatient_registration.event.id)

        return True

    def write(self, ids, values):
        event_obj = Pool().get('calendar.event')
        patient_obj = Pool().get('gnuhealth.patient')
        hospital_bed_obj = Pool().get('gnuhealth.hospital.bed')

        if isinstance(ids, (int, long)):
            ids = [ids]
        for inpatient_registration_id in ids:
            inpatient_registration = self.browse(inpatient_registration_id)
            if inpatient_registration.event:
                if 'hospitalization_date' in values:
                    event_obj.write(inpatient_registration.event.id, {
                        'dtstart': values['hospitalization_date'],
                        })
                if 'discharge_date' in values:
                    event_obj.write(inpatient_registration.event.id, {
                        'dtend': values['discharge_date'],
                        })
                if 'bed' in values:
                    bed = hospital_bed_obj.browse(values['bed'])
                    event_obj.write(inpatient_registration.event.id, {
                        'calendar': bed.calendar.id,
                        })
                if 'patient' in values:
                    patient = patient_obj.browse(values['patient'])
                    event_obj.write(inpatient_registration.event.id, {
                        'summary': patient.name.name,
                        })
        return super(InpatientRegistration, self).write(ids, values)

    def delete(self, ids):
        event_obj = Pool().get('calendar.event')

        if isinstance(ids, (int, long)):
            ids = [ids]
        for inpatient_registration_id in ids:
            inpatient_registration = self.browse(inpatient_registration_id)
            if inpatient_registration.event:
                event_obj.delete(inpatient_registration.event.id)
        return super(InpatientRegistration, self).delete(ids)

InpatientRegistration()



# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    MODULE : Socioeconomics
#    Copyright (C) 2008-2012  Luis Falcon <lfalcon@gnusolidario.org>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
{
    'name': 'GNU Health : Socioeconomics',
    'version': '1.6.3',
    'author': 'GNU Solidario',
    'email': 'health@gnusolidario.org',
    'website': 'http://health.gnu.org',
    'depends': [
        'health',
    ],
    'name_es_ES': 'GNU Health : Socioeconomía',
    'translation': [
    ],

    'description': '''


This module takes care of the input of all the socio-economic factors that influence the health of the individual / family and society.

Among others, we include the following factors :

- Living conditions
- Educational level
- Infrastructure (electricity, sewers, ... )
- Family affection ( APGAR )
- Drug addiction
- Hostile environment
- Teenage Pregnancy
- Working children


''',
    'description_es_ES': '''
Este módulo se encarge de ingresar y procesar los factores socio-económicos que influyen en la salud del individuo, su familia y sociedad.

Algunos aspectos que abarca :

- Condiciones de vida
- Nivel educativo
- Infraestructura (electricidad, gas, alcantarillas... )
- Afectividad familiar
- Trabajo Infantil
- Entornos hostiles
- Embarazos adolescentes
- Profesiones
- Violencia doméstica

''',

    'xml': [
        'health_socioeconomics_view.xml',
    ],
    'active': False,
}



# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2012  Luis Falcon <lfalcon@gnusolidario.org>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from trytond.model import ModelView
from trytond.wizard import Wizard, StateTransition, StateView, StateTransition, \
    Button
from trytond.transaction import Transaction

from trytond.pool import Pool

class CreateLabTestOrderInit(ModelView):
    'Create Test Report Init'
    _name = 'gnuhealth.lab.test.create.init'
    _description = __doc__

CreateLabTestOrderInit()


class CreateLabTestOrder(Wizard):
    'Create Lab Test Report'
    _name = 'gnuhealth.lab.test.create'

    start = StateView('gnuhealth.lab.test.create.init',
        'health_lab.view_lab_make_test', [
            Button('Cancel', 'end', 'tryton-cancel'),
            Button('Create Test Order', 'create_lab_test', 'tryton-ok', True),
            ])
    
    create_lab_test = StateTransition()


    def transition_create_lab_test(self, session):
        test_request_obj = Pool().get('gnuhealth.patient.lab.test')
        lab_obj = Pool().get('gnuhealth.lab')

        test_report_data = {}
        test_cases = []
        
        test_obj = test_request_obj.browse(Transaction().context.get('active_ids'))
        
        for lab_test_order in test_obj:
                  
            if lab_test_order.state == 'ordered':
                raise Exception('The Lab test order is already created.')

            test_report_data['test'] = lab_test_order.name.id
            test_report_data['patient'] = lab_test_order.patient_id.id
            test_report_data['requestor'] = lab_test_order.doctor_id.id
            test_report_data['date_requested'] = lab_test_order.date

            for critearea in lab_test_order.name.critearea:
                test_cases.append(('create', {
                        'name': critearea.name,
                        'sequence': critearea.sequence,
                        'lower_limit': critearea.lower_limit,
                        'upper_limit': critearea.upper_limit,                    
                        'normal_range': critearea.normal_range,
                        'units': critearea.units.id,
                    }))
            test_report_data['critearea'] = test_cases
            lab_id = lab_obj.create(test_report_data)
            test_request_obj.write(lab_test_order.id, {'state': 'ordered'})
                       
        return 'end'
        

CreateLabTestOrder()



# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2012  Luis Falcon <lfalcon@gnusolidario.org>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from datetime import datetime
from trytond.model import ModelView, ModelSingleton, ModelSQL, fields
from trytond.transaction import Transaction

from trytond.pool import Pool


class GnuHealthSequences(ModelSingleton, ModelSQL, ModelView):
    "Standard Sequences for GNU Health"

    _description = __doc__
    _name = "gnuhealth.sequences"

    lab_sequence = fields.Property(fields.Many2One('ir.sequence',
        'Lab Sequence', domain=[('code', '=', 'gnuhealth.lab')],
        required=True))

GnuHealthSequences()

class PatientData(ModelSQL, ModelView):
    'Patient lab tests'
    _name = 'gnuhealth.patient'
    _description = __doc__

    lab_test_ids = fields.One2Many('gnuhealth.patient.lab.test', 'patient_id',
        'Lab Tests Required')

PatientData()


class TestType(ModelSQL, ModelView):
    'Type of Lab test'
    _name = 'gnuhealth.lab.test_type'
    _description = __doc__

    name = fields.Char('Test', 
        help="Test type, eg X-Ray, hemogram,biopsy...", required=True, select=True)
    code = fields.Char('Code', 
        help="Short name - code for the test", required=True, select=True)
    info = fields.Text('Description')
    product_id = fields.Many2One('product.product', 'Service', required=True)
    critearea = fields.One2Many('gnuhealth.lab.test.critearea', 'test_type_id',
        'Test Cases')

    def __init__(self):
        super(TestType, self).__init__()
        self._sql_constraints = [
            ('code_uniq', 'unique(name)', 'The Lab Test code must be unique'),
        ]

TestType()


class Lab(ModelSQL, ModelView):
    'Lab Test'
    _name = 'gnuhealth.lab'
    _description = __doc__

    name = fields.Char('ID', help="Lab result ID",readonly=True)
    test = fields.Many2One('gnuhealth.lab.test_type', 'Test type',
        help="Lab test type", required=True, select=True)
    patient = fields.Many2One('gnuhealth.patient', 'Patient',
     help="Patient ID", required=True, select=True)
    pathologist = fields.Many2One('gnuhealth.physician', 'Pathologist',
        help="Pathologist", select=True)
    requestor = fields.Many2One('gnuhealth.physician', 'Physician',
        help="Doctor who requested the test", select=True)
    results = fields.Text('Results')
    diagnosis = fields.Text('Diagnosis')
    critearea = fields.One2Many('gnuhealth.lab.test.critearea', 'gnuhealth_lab_id',
        'Lab Test Critearea')
    date_requested = fields.DateTime('Date requested', required=True, select=True)
    date_analysis = fields.DateTime('Date of the Analysis', select=True)

    def __init__(self):
        super(Lab, self).__init__()
        self._sql_constraints += [
            ('id_uniq', 'unique (name)', 'The test ID code must be unique'),
        ]

    def default_date_requested(self):
        return datetime.now()

    def default_analysis(self):
        return datetime.now()

    def create(self, values):
        sequence_obj = Pool().get('ir.sequence')
        config_obj = Pool().get('gnuhealth.sequences')

        values = values.copy()
        if not values.get('name'):
            config = config_obj.browse(1)
            values['name'] = sequence_obj.get_id(
            config.lab_sequence.id)

        return super(Lab, self).create(values)

Lab()


class GnuHealthLabTestUnits(ModelSQL, ModelView):
    'Lab Test Units'
    _name = 'gnuhealth.lab.test.units'
    _description = __doc__

    name = fields.Char('Unit', select=True)
    code = fields.Char('Code', select=True)

    def __init__(self):
        super(GnuHealthLabTestUnits, self).__init__()
        self._sql_constraints = [
            ('name_uniq', 'unique(name)', 'The Unit name must be unique'),
        ]

GnuHealthLabTestUnits()


class GnuHealthTestCritearea(ModelSQL, ModelView):
    'Lab Test Critearea'
    _name = 'gnuhealth.lab.test.critearea'
    _description = __doc__

    name = fields.Char('Analyte', required=True, select=True)
    excluded = fields.Boolean('Excluded',help='Select this option when' \
        ' this analyte is excluded from the test')
    result = fields.Float('Value')
    result_text = fields.Char('Result - Text',help='Non-numeric results. For example '\
        'qualitative values, morphological, colors ...')
    remarks = fields.Char('Remarks')
    normal_range = fields.Text('Reference')
    lower_limit = fields.Float ('Lower Limit')
    upper_limit = fields.Float ('Upper Limit')
    warning = fields.Boolean('Warn',help='Warns the patient about this analyte result' \
        ' It is useful to contextualize the result to each patient status ' \
        ' like age, sex, comorbidities, ...',
         on_change_with=['result', 'lower_limit', 'upper_limit'])
    units = fields.Many2One('gnuhealth.lab.test.units', 'Units')
    test_type_id = fields.Many2One('gnuhealth.lab.test_type', 'Test type',
     select=True)
    gnuhealth_lab_id = fields.Many2One('gnuhealth.lab', 'Test Cases', select=True)
    sequence = fields.Integer('Sequence')

    def __init__(self):
        super(GnuHealthTestCritearea, self).__init__()
        self._order.insert(0, ('sequence', 'ASC'))

    def default_sequence(self):
        return 1

    def default_excluded(self):
        return False
        
    def on_change_with_warning(self, vals):
        lower_limit = vals.get('lower_limit')
        upper_limit = vals.get('upper_limit')
        result = vals.get ('result')
        if (result < lower_limit or result > upper_limit):
            warning = True
        else:
            warning = False
        return warning
        

GnuHealthTestCritearea()


class GnuHealthPatientLabTest(ModelSQL, ModelView):
    'Patient Lab Test'
    _name = 'gnuhealth.patient.lab.test'
    _description = __doc__

    name = fields.Many2One('gnuhealth.lab.test_type', 'Test Type', required=True,
     select=True)
    date = fields.DateTime('Date', select=True)
    state = fields.Selection([
        ('draft', 'Draft'),
        ('tested', 'Tested'),
        ('ordered', 'Ordered'),       
        ('cancel', 'Cancel'),
        ], 'State', readonly=True, select=True)
    patient_id = fields.Many2One('gnuhealth.patient', 'Patient', required=True,
     select=True)
    doctor_id = fields.Many2One('gnuhealth.physician', 'Doctor',
        help="Doctor who Request the lab test.", select=True)

    def default_date(self):
        return datetime.now()

    def default_state(self):
        return 'draft'

    def default_doctor_id(self):
        user_obj = Pool().get('res.user')
        user = user_obj.browse(Transaction().user)
        uid = int(user.id)

        party_id = Pool().get('party.party').search([
                ('internal_user', '=', uid)])
        if party_id:
            dr_id = Pool().get('gnuhealth.physician').search([
                    ('name', '=', party_id[0])])
            if dr_id:
                return dr_id[0]
            else:
                raise Exception('There is no physician defined ' \
                                'for current user.')
        else:
            return False

GnuHealthPatientLabTest()



# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2012  Luis Falcon <lfalcon@gnusolidario.org>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from trytond.model import ModelView, ModelSQL, fields
from datetime import datetime
from trytond.pool import Pool
from trytond.transaction import Transaction


# Class : PatientRounding
# Assess the patient and evironment periodically
# Usually done by nurses

class PatientRounding(ModelSQL, ModelView):
    'Patient Rounding'
    _name = 'gnuhealth.patient.rounding'
    _description = __doc__

    name = fields.Many2One('gnuhealth.inpatient.registration', 'Registration Code', required=True)

    health_professional = fields.Many2One('gnuhealth.physician', 'Health Professional', readonly=True)

    evaluation_start = fields.DateTime('Start', required=True)
    evaluation_end = fields.DateTime('End', required=True)

    environmental_assessment = fields.Char('Environment', help="Environment" \
        " assessment . State any disorder in the room.")

    # The 6 P's of rounding
    pain = fields.Boolean('Pain', help="Check if the patient is in pain")
    pain_level = fields.Integer('Pain', help="Enter the pain level, from 1 to 10")
    potty = fields.Boolean ('Potty', help="Check if the patient needs to urinate / defecate")
    position = fields.Boolean ('Position', help="Check if the patient needs to be repositioned or is unconfortable")
    proximity = fields.Boolean ('Proximity', help="Check if personal items, water, alarm, ... are not in easy reach")
    pump = fields.Boolean ('Pumps', help="Check if there is any issues with the pumps - IVs ... ")
    personal_needs = fields.Boolean ('Personal needs',help="Check if the patient requests anything")

    # Vital Signs
    systolic = fields.Integer('Systolic Pressure')
    diastolic = fields.Integer('Diastolic Pressure')
    bpm = fields.Integer('Heart Rate',
        help='Heart rate expressed in beats per minute')
    respiratory_rate = fields.Integer('Respiratory Rate',
        help='Respiratory rate expressed in breaths per minute')
    osat = fields.Integer('Oxygen Saturation',
        help='Oxygen Saturation(arterial).')
    temperature = fields.Float('Temperature',
        help='Temperature in celsius')


    #Glycemia
    glycemia = fields.Integer('Glycemia', help='Blood Glucose level')

    depression = fields.Boolean ('Depression signs',help="Check this if the patient shows signs of depression")

    evolution = fields.Selection([
        ('n', 'Status Quo'),
        ('i', 'Improving'),
        ('w', 'Worsening'),
        ], 'Evolution', required=True, help="Check your judgement of current patient condition", sort=False)

    round_summary = fields.Text('Round Summary')

    warning = fields.Boolean('Warning', help="Check this box to alert the supervisor about this patient rounding" \
        ". It will be shown in red in the rounding list")

    procedures = fields.One2Many('gnuhealth.rounding_procedure', 'name', 'Procedures',
        help="List of the procedures in this rounding. Please enter the first " \
        "one as the main procedure")

    def default_health_professional(self):
        cursor = Transaction().cursor
        user_obj = Pool().get('res.user')
        user = user_obj.browse(Transaction().user)
        login_user_id = int(user.id)
        cursor.execute('SELECT id FROM party_party WHERE is_doctor=True AND \
            internal_user = %s LIMIT 1', (login_user_id,))
        partner_id = cursor.fetchone()
        if not partner_id:
            self.raise_user_error('No health professional associated to this \
                user')
        else:
            cursor = Transaction().cursor
            cursor.execute('SELECT id FROM gnuhealth_physician WHERE \
                name = %s LIMIT 1', (partner_id[0],))
            doctor_id = cursor.fetchone()

            return int(doctor_id[0])


    def default_evaluation_start(self):
        return datetime.now()

PatientRounding()


class RoundingProcedure(ModelSQL, ModelView):
    'Rounding - Procedure'
    _name = 'gnuhealth.rounding_procedure'
    _description = __doc__

    name = fields.Many2One('gnuhealth.patient.rounding', 'Rounding')
    procedure = fields.Many2One('gnuhealth.procedure', 'Code', required=True,
        select=True,
        help="Procedure Code, for example ICD-10-PCS Code 7-character string")
    notes = fields.Text('Notes')

RoundingProcedure()



#!/usr/bin/env python
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2012  Sebastian Marro <smarro@gnusolidario.org>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
import sys, os
DIR = os.path.abspath(os.path.normpath(os.path.join(__file__,
    '..', '..', '..', '..', '..', 'trytond')))
if os.path.isdir(DIR):
    sys.path.insert(0, os.path.dirname(DIR))

import unittest
import trytond.tests.test_tryton
from trytond.tests.test_tryton import test_view, test_depends


class HealthServicesTestCase(unittest.TestCase):
    '''
    Test HealthServices module.
    '''

    def setUp(self):
        trytond.tests.test_tryton.install_module('health_services')

    def test0005views(self):
        '''
        Test views.
        '''
        test_view('health_services')

    def test0006depends(self):
        '''
        Test depends.
        '''
        test_depends()

def suite():
    suite = trytond.tests.test_tryton.suite()
    suite.addTests(unittest.TestLoader().loadTestsFromTestCase(
        HealthServicesTestCase))
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(suite())
